/* lib/x509/x509_sgn.c */
/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
 * All rights reserved.
 * 
 * This file is part of an SSL implementation written
 * by Eric Young (eay@mincom.oz.au).
 * The implementation was written so as to conform with Netscapes SSL
 * specification.  This library and applications are
 * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 * as long as the following conditions are aheared to.
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.  If this code is used in a product,
 * Eric Young should be given attribution as the author of the parts used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Eric Young (eay@mincom.oz.au)
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "crypto.h"
#include "der.h"
#include "bn.h"
#include "x509.h"
#include "x509_obj.h"
#include "buffer.h"
#include "pem.h"

int X509_sign(x, rsa, type)
X509 *x;
RSA *rsa;
int type;
	{
	unsigned char *p,*buf_in,*buf_out;
	int inl,outl;
	PEM_CTX ctx;

	inl=i2D_X509_CINF(x->cert_info,NULL);
	buf_in=malloc((unsigned int)inl);
	outl=RSA_size(rsa);
	buf_out=malloc((unsigned int)outl);
	if ((buf_in == NULL) || (buf_out == NULL))
		{ X509err(X509_F_X509_SIGN,ERR_R_MALLOC_FAILURE); return(0); }
	p=buf_in;
	i2D_X509_CINF(x->cert_info,&p);
	if (	(!PEM_SignInit(&ctx,type)) ||
		(!PEM_SignUpdate(&ctx,(unsigned char *)buf_in,inl)) ||
		(!PEM_SignFinal(&ctx,(unsigned char *)buf_out,
			(unsigned int *)&outl,rsa)))
		{ X509err(X509_F_X509_SIGN,ERR_R_PEM_LIB); return(0); }

	x->sig_alg->parameters->length=0;
	DER_OBJECT_free(x->sig_alg->algorithm);
	x->sig_alg->algorithm=X509_dup_DER_OBJECT(X509_nid2obj(type));
	x->sig_alg->type=DER_NULL;
	if (x->sig_alg->algorithm == NULL) return(0);
		
	if (x->signature->data != NULL) free(x->signature->data);
	x->signature->data=buf_out;
	x->signature->length=outl;
	memset(buf_in,0,(unsigned int)inl);
	free(buf_in);
	return(outl);
	}
